/*	
  Copyright(c) 2011 Christoph Schmalz <christoph.schmalz@informatik.uni-erlangen.de>
  
  This file is licensed under the terms of of the GNU General Public License, 
  version 3, as published by the Free Software Foundation. You can find it at:
  http://www.gnu.org/licenses/gpl.html
*/

#ifndef __UTIL_H__
#define __UTIL_H__

#include <vector>

#define _USE_MATH_DEFINES
#include "math.h"

#include "opencv2/opencv.hpp"
using namespace cv;

#ifndef M_2PI
	#define M_2PI 6.2831853071795864769252867665590057683943387987502116419498
#endif

#ifndef M_2PIf
	#define M_2PIf 6.2831853071795864769252867665590057683943387987502116419498f
#endif

#ifndef M_PIf
	#define M_PIf 3.1415926535897932384626433832795028841971693993751058209749f
#endif

#define D2R(x) ( (x)*M_PI/180 )
#define R2D(x) ( (x)*180/M_PI )
#define D2Rf(x) ( (x)*M_PIf/180 )
#define R2Df(x) ( (x)*180/M_PIf )
#define SQR(x) ( (x)*(x) )

typedef std::vector<Mat*>	imageArray_t;
typedef unsigned char		uint8_t;
typedef unsigned long		uint32_t;

///////////////////////////////////////////////////////////////////////////////
// Class: CVec2D
// Description:
///////////////////////////////////////////////////////////////////////////////

struct CVec2D
{
	double		x, y;

	//-----------------

				CVec2D(double dX=0, double dY=0) : x(dX), y(dY) {}

				CVec2D(const CVec2D &v) : x(v.x), y(v.y) {}

	void		Set(double dX=0, double dY=0) {x=dX; y=dY;}

	// Scalar product
	inline double		Dot(const CVec2D &v) const 
					{ return (x * v.x + y * v.y); }

	// Cross product (most useful definition in 2d)
	inline double		Cross(const CVec2D &v) const 
					{ return x*v.y - y*v.x; }

	inline double		GetLengthSq() const { return x*x+y*y; }
	inline double		GetLength() const { return sqrt(GetLengthSq()); }

	const CVec2D		operator+(const CVec2D& rhs) const { return CVec2D(x+rhs.x,y+rhs.y); }
	const CVec2D		operator-(const CVec2D& rhs) const { return CVec2D(x-rhs.x,y-rhs.y); }
	const CVec2D		operator*(const double rhs) const { return CVec2D(x*rhs,y*rhs); }

	CVec2D&	Normalize() 
					{ double dLength = GetLength(); 
					  if (dLength==0) return *this;
					  else {x/=dLength; y/=dLength; return *this; } }
};


///////////////////////////////////////////////////////////////////////////////
// Class: CVec3D
// Description:
///////////////////////////////////////////////////////////////////////////////

struct CVec3D
{
	double	x,y,z;

	//---------------------------
	
				CVec3D(double dX=0, double dY=0, double dZ=0) :
				  x(dX), y(dY), z(dZ) {}

				CVec3D(const CVec3D& v) :
				  x(v.x), y(v.y), z(v.z) {}

	void		Set(double dX=0, double dY=0, double dZ=0) 
					{ x=dX; y=dY; z=dZ; }

	inline CVec3D		Cross(const CVec3D& other) const
					{ 	return CVec3D( y*(other.z) - z*(other.y),
										z*(other.x) - x*(other.z),
										x*(other.y) - y*(other.x) ); }

	inline double		Dot(const CVec3D& other) const
					{	return x*other.x+y*other.y+z*other.z; }
	
	inline double		GetLengthSq() const { return x*x+y*y+z*z; }
	inline double		GetLength() const { return sqrt(GetLengthSq()); }

	CVec3D&	Normalize() 
					{ double dLength = GetLength(); 
					  if (dLength==0) return *this;
					  else {x/=dLength; y/=dLength; z/=dLength; return *this; } }
};

///////////////////////////////////////////////////////////////////////////////
// Class: CCalibMark
///////////////////////////////////////////////////////////////////////////////

struct CCalibMark
{
	// The world co-ordinates
	CVec3D	world;

	// The image co-ordinates
	CVec2D	img;

	// quality
	float	q;

	// id
	size_t	id;

	//---------------

		CCalibMark() {q=1;id=0;}

		CCalibMark(const CVec3D& worldCoords, 
			const CVec2D& imageCoords,
			float q_=1, size_t id_=0) :
			img(imageCoords), world(worldCoords), q(q_), id(id_) {}
};


///////////////////////////////////////////////////////////////////////////////
// Class: CLine2D
// Description:
///////////////////////////////////////////////////////////////////////////////

struct CLine2D
{
	CVec2D p, v; //point and direction

	//-----------------

	CLine2D() {}
	CLine2D(const CVec2D& p_, const CVec2D& v_) : p(p_), v(v_) {}

	bool	Intersect(const CLine2D& other, CVec2D& intersection);
};
 
///////////////////////////////////////////////////////////////////////////////
// Class: CLine3D
// Description:
///////////////////////////////////////////////////////////////////////////////

struct CLine3D
{
	CVec3D p, v; //point and direction

	//-----------------

	CLine3D() {}
	CLine3D(const CVec3D& p_, const CVec3D& v_) : p(p_), v(v_) {}
};



///////////////////////////////////////////////////////////////////////////////
// Class: CPlane3D
// Description:
//	plane in ax+by+cz+d=0 form
///////////////////////////////////////////////////////////////////////////////

struct CPlane3D
{
	// The normal of the plane
	CVec3D				normal;	
	// The D parameter of the plane
	double				d;

	//----------------------------

	CPlane3D() {}	
	CPlane3D(const CVec3D& Normal, const double dD) : 
		normal(Normal), d(dD) {}

	void				Set(const CVec3D& Normal, const double dD)
									{ normal=Normal; d=dD; }

	bool				Intersect(	const CPlane3D&		plane,
									CLine3D&			point ) const;
};


///////////////////////////////////////////////////////////////////////////////
// Function: BinaryToGrayCode
///////////////////////////////////////////////////////////////////////////////

inline unsigned int // The Gray code 
 BinaryToGrayCode(unsigned int binary) //in: binary code to be converted
{
    return (binary ^ (binary>>1));
}


///////////////////////////////////////////////////////////////////////////////
// Function: GrayCodeToBinary
///////////////////////////////////////////////////////////////////////////////

inline unsigned int // The standard binary number 
 GrayCodeToBinary(unsigned int graycode) //in: gray code to be converted
{
	unsigned int binary=graycode, shift=1, tmp;
	while(1)
	{
		tmp = (binary >> shift);
		binary ^= tmp;

		if (tmp<=1 || shift==16) 
			return binary;

		shift = shift<<1;
	}
}

#endif //__UTIL_H__